/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.swing;

import java.awt.*;
import net.java.truelicense.core.LicenseConsumerManager;
import static net.java.truelicense.swing.Messages.*;
import net.java.truelicense.swing.nexes.Wizard;

/**
 * This is the internationalized license management wizard class.
 * It can be used to install and verify licenses visually for the
 * application's user.
 *
 * @author Christian Schlichtherle
 */
public class LicenseWizard extends Wizard {

    private WelcomePanel.Descriptor welcome;

    public LicenseWizard(LicenseConsumerManager manager) {
        init(manager);
    }

    public LicenseWizard(final LicenseConsumerManager manager, Dialog owner) {
        super(owner);
        init(manager);
    }

    public LicenseWizard(final LicenseConsumerManager manager, Frame owner) {
        super(owner);
        init(manager);
    }

    private void init(final LicenseConsumerManager cm) {
        super.setDialogTitle(message("LicenseWizard.title",
                cm.subject()).toString());
        final ObservableLicenseConsumerManager ocm = new ObservableLicenseConsumerManager(cm);
        super.addPanelDescriptor(welcome = new WelcomePanel.Descriptor(ocm));
        super.addPanelDescriptor(new InstallPanel.Descriptor(ocm));
        super.addPanelDescriptor(new LicensePanel.Descriptor(ocm));
        super.addPanelDescriptor(new UninstallPanel.Descriptor(ocm));
        super.setCurrentPanelDescriptor(welcome);
    }

    /**
     * Returns {@code true} if and only if the uninstall button is visible.
     * By default, this button is <em>not</em> visible.
     */
    public boolean isUninstallButtonVisible() {
        return welcome.isUninstallButtonVisible();
    }

    /** Sets the visibility of the uninstall button. */
    public void setUninstallButtonVisible(boolean visible) {
        welcome.setUninstallButtonVisible(visible);
    }
}
